@@ -103,10 +103,15 @@ class RobotFrameworkServerApi(PythonLanguageServer):
 
     def _threaded_lint(self, doc_uri, monitor: IMonitor):
         from robocorp_ls_core.jsonrpc.exceptions import JsonRpcRequestCancelled
+        from robotframework_ls.impl.robot_lsp_constants import (
+            OPTION_ROBOT_LINT_ROBOCOP_ENABLED,
+        )
+        from robocorp_ls_core import uris
 
         try:
             from robotframework_ls.impl.ast_utils import collect_errors
             from robotframework_ls.impl import code_analysis
+            import os.path
 
             log.debug("Lint: starting (in thread).")
 
@@ -114,15 +119,48 @@ class RobotFrameworkServerApi(PythonLanguageServer):
             if completion_context is None:
                 return []
 
+            config = completion_context.config
+            robocop_enabled = config is None or config.get_setting(
+                OPTION_ROBOT_LINT_ROBOCOP_ENABLED, bool, True
+            )
+
             ast = completion_context.get_ast()
+            source = completion_context.doc.source
             monitor.check_cancelled()
             errors = collect_errors(ast)
             log.debug("Collected AST errors (in thread): %s", len(errors))
             monitor.check_cancelled()
             analysis_errors = code_analysis.collect_analysis_errors(completion_context)
+            monitor.check_cancelled()
             log.debug("Collected analysis errors (in thread): %s", len(analysis_errors))
             errors.extend(analysis_errors)
-            return [error.to_lsp_diagnostic() for error in errors]
+
+            lsp_diagnostics = [error.to_lsp_diagnostic() for error in errors]
+
+            try:
+                if robocop_enabled:
+                    from robocorp_ls_core.robocop_wrapper import (
+                        collect_robocop_diagnostics,
+                    )
+
+                    workspace = completion_context.workspace
+                    if workspace is not None:
+                        project_root = workspace.root_path
+                    else:
+                        project_root = os.path.abspath(".")
+
+                    monitor.check_cancelled()
+                    lsp_diagnostics.extend(
+                        collect_robocop_diagnostics(
+                            project_root, ast, uris.to_fs_path(doc_uri), source
+                        )
+                    )
+            except Exception:
+                log.exception(
+                    "Error collecting Robocop errors (possibly an unsupported Robocop version is installed)."
+                )
+
+            return lsp_diagnostics
         except JsonRpcRequestCancelled:
             raise JsonRpcRequestCancelled("Lint cancelled (inside lint)")
         except:
@@ -280,7 +318,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         return [x.to_dict() for x in create_text_edit_from_diff(text, new_contents)]
 
     def _create_completion_context(
-        self, doc_uri, line, col, monitor: Union[IMonitor, None]
+        self, doc_uri, line, col, monitor: Optional[IMonitor]
     ):
         from robotframework_ls.impl.completion_context import CompletionContext
 
@@ -348,13 +386,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         )
 
     def m_text_document__semantic_tokens__range(self, textDocument=None, range=None):
-        from robotframework_ls.impl.semantic_tokens import semantic_tokens_range
-
-        doc_uri = textDocument["uri"]
-        context = self._create_completion_context(doc_uri, -1, -1, None)
-        if context is None:
-            return []
-        return semantic_tokens_range(context, range)
+        raise RuntimeError("Not currently implemented!")
 
     def m_text_document__semantic_tokens__full(self, textDocument=None):
         from robotframework_ls.impl.semantic_tokens import semantic_tokens_full
@@ -362,7 +394,7 @@ class RobotFrameworkServerApi(PythonLanguageServer):
         doc_uri = textDocument["uri"]
         context = self._create_completion_context(doc_uri, -1, -1, None)
         if context is None:
-            return []
+            return {"resultId": None, "data": []}
         return {"resultId": None, "data": semantic_tokens_full(context)}
 
     def m_shutdown(self, **_kwargs):
